'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.operators = exports.dotpath = undefined;

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getOwnPropertyDescriptor = require('babel-runtime/core-js/object/get-own-property-descriptor');

var _getOwnPropertyDescriptor2 = _interopRequireDefault(_getOwnPropertyDescriptor);

var _getOwnPropertyNames = require('babel-runtime/core-js/object/get-own-property-names');

var _getOwnPropertyNames2 = _interopRequireDefault(_getOwnPropertyNames);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _defineProperty = require('babel-runtime/core-js/object/define-property');

var _defineProperty2 = _interopRequireDefault(_defineProperty);

exports.humanize = humanize;
exports.titleize = titleize;
exports.classify = classify;
exports.tableize = tableize;
exports.underscore = underscore;
exports.parameterize = parameterize;
exports.slugify = slugify;
exports.singularize = singularize;
exports.pluralize = pluralize;
exports.lazy = lazy;
exports.flatten = flatten;
exports.singularize = singularize;
exports.createDelegators = createDelegators;
exports.values = values;
exports.mixin = mixin;
exports.access = access;
exports.copyProp = copyProp;
exports.runWithLocals = runWithLocals;
exports.noConflict = noConflict;
exports.carve = carve;
exports.isDomain = isDomain;
exports.isPromise = isPromise;
exports.isArray = isArray;
exports.isRegex = isRegex;
exports.filterQuery = filterQuery;
exports.abort = abort;
exports.findPackageSync = findPackageSync;
exports.findPackage = findPackage;
exports.splitPath = splitPath;
exports.skypagerBabel = skypagerBabel;
exports.pathExists = pathExists;
exports.findPackageManifest = findPackageManifest;
exports.loadManifestFromDirectory = loadManifestFromDirectory;

var _path = require('path');

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _utile = require('utile');

var _utile2 = _interopRequireDefault(_utile);

var _lodash = require('lodash');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var inflections = _utile2.default.inflect;
var DOMAIN_REGEX = /^[a-zA-Z0-9_-]+\.[.a-zA-Z0-9_-]+$/;

var dotpath = exports.dotpath = {
  set: _lodash.set, get: _lodash.get
};

var equals = _lodash.eq;
var not_equals = function not_equals() {
  return !_lodash.eq.apply(undefined, arguments);
};
var neq = function neq() {
  return !_lodash.eq.apply(undefined, arguments);
};

var operators = exports.operators = {
  lt: _lodash.lt, gt: _lodash.gt, gte: _lodash.gte, lte: _lodash.lte, eq: _lodash.eq, neq: neq, equals: equals, not_equals: not_equals
};

module.exports.assign = _objectAssign2.default;
module.exports.defaults = _lodash.defaults;
module.exports.pick = _lodash.pick;
module.exports.some = _lodash.some;
module.exports.any = _lodash.some;
module.exports.result = _lodash.result;
module.exports.clone = _lodash.cloneDeep;
module.exports.template = _lodash.template;
module.exports.isObject = _lodash.isObject;
module.exports.isNumber = _lodash.isNumber;
module.exports.isString = _lodash.isString;

var hidden = {
  getter: function getter(target, name, fn) {
    var configurable = arguments.length <= 3 || arguments[3] === undefined ? false : arguments[3];

    (0, _defineProperty2.default)(target, name, {
      configurable: configurable,
      enumerable: false,
      get: function get() {
        return typeof fn === 'function' ? fn.call(target) : fn;
      }
    });
  },

  property: function property(target, name, value) {
    var configurable = arguments.length <= 3 || arguments[3] === undefined ? false : arguments[3];

    (0, _defineProperty2.default)(target, name, {
      configurable: configurable,
      enumerable: false,
      value: value
    });
  }
};

module.exports.hidden = hidden;
module.exports.hide = hidden;

function humanize(s) {
  return inflections.humanize(s).replace(/-|_/g, ' ');
}

function titleize(s) {
  return inflections.titleize(humanize(s));
}

function classify(s) {
  return inflections.classify(s);
}

function tableize(s) {
  return inflections.tableize(s);
}

function underscore(s) {
  return tableize(s);
}

function parameterize(s) {
  s = s.replace(/\\|\//g, '-', '');
  s = s.replace(/[^-\w\s]/g, ''); // remove unneeded chars
  s = s.replace(/^\s+|\s+$/g, ''); // trim leading/trailing spaces
  s = s.replace(/[-\s]+/g, '-'); // convert spaces to hyphens
  s = s.toLowerCase(); // convert to lowercase
  return s;
}

function slugify(s) {
  return parameterize(s);
}

function singularize(word) {
  return inflections.singularize(word);
}

function pluralize(word) {
  return inflections.pluralize(word);
}

function lazy(target, attribute, fn) {
  var enumerable = arguments.length <= 3 || arguments[3] === undefined ? false : arguments[3];

  (0, _defineProperty2.default)(target, attribute, {
    configurable: true,
    enumerable: enumerable,
    get: function get() {
      delete target[attribute];

      if (enumerable) {
        return target[attribute] = fn.call(target);
      } else {
        var value = fn.call(target);

        (0, _defineProperty2.default)(target, attribute, {
          enumerable: false,
          configurable: true,
          value: value
        });

        return value;
      }
    }
  });
}

function flatten(array) {
  return array.reduce(function (a, b) {
    return a.concat(b);
  }, []);
}

function singularize(string) {
  return inflections.singularize(string);
}

function createDelegators(target, source) {
  var options = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var excludeKeys = options.exclude || options.except || [];
  var sourceKeys = (0, _keys2.default)(source).filter(function (key) {
    return excludeKeys.indexOf(key) === -1;
  });

  sourceKeys.forEach(function (key) {
    return (0, _defineProperty2.default)(target, key, {
      get: function get() {
        return source[key];
      }
    });
  });
}

function values(object) {
  return (0, _keys2.default)(object).map(function (key) {
    return object[key];
  });
}

function mixin(target, source) {
  target = target.prototype;source = source.prototype;

  (0, _getOwnPropertyNames2.default)(source).forEach(function (name) {
    if (name !== 'constructor') (0, _defineProperty2.default)(target, name, (0, _getOwnPropertyDescriptor2.default)(source, name));
  });
}

function access(object, dotted) {
  return dotted.split('.').reduce(function (memo, current) {
    return memo[current];
  }, object);
}

function copyProp(property) {
  return {
    from: function from(source) {
      return {
        to: function to(target) {
          var desc = (0, _getOwnPropertyDescriptor2.default)(source, property);

          if (typeof desc !== 'undefined' && desc.configurable) {
            (0, _defineProperty2.default)(target, property, desc);
          }
        }
      };
    }
  };
}

function runWithLocals(fn) {
  var locals = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  return new _promise2.default(function (resolve, reject) {
    noConflict(fn, locals, undefined, resolve, reject)();
  });
}

function noConflict(fn) {
  var provider = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var scope = arguments[2];
  var onSuccess = arguments[3];
  var onError = arguments[4];

  invariant(fn, 'provide a function');
  invariant(provider, 'provide a provider');

  var safe = {};

  return function () {
    (0, _keys2.default)(provider).forEach(function (globalProp) {
      if (global.hasOwnProperty(globalProp)) {
        var descriptor = (0, _getOwnPropertyDescriptor2.default)(global, globalProp);

        if (descriptor && descriptor.configurable) {
          (0, _defineProperty2.default)(safe, globalProp, descriptor);
        }
      }

      if (onSuccess) {}

      (0, _defineProperty2.default)(global, globalProp, (0, _getOwnPropertyDescriptor2.default)(provider, globalProp));
    });

    var result = undefined;
    var err = undefined;

    try {
      if (onSuccess) {}

      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      result = scope ? fn.call.apply(fn, [scope].concat(args)) : fn.apply(undefined, args);
    } catch (e) {
      err = e;
    }

    (0, _keys2.default)(provider).forEach(function (remove) {
      return delete global[remove];
    });

    (0, _keys2.default)(safe).forEach(function (restore) {
      delete global[restore];
      (0, _defineProperty2.default)(global, restore, (0, _getOwnPropertyDescriptor2.default)(safe, restore));
    });

    if (err) {
      typeof onError === 'function' && onError(err);
      throw err;
    }

    typeof onSuccess === 'function' && onSuccess(err);
    return result;
  };
}

function carve(dataPath, resultValue) {
  var initialValue = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  dotpath.set(initialValue, dataPath, resultValue);
  return initialValue;
}

function isDomain(value) {
  return value.match(DOMAIN_REGEX);
}

function isPromise(obj) {
  return !!obj && ((typeof obj === 'undefined' ? 'undefined' : (0, _typeof3.default)(obj)) === 'object' || typeof obj === 'function') && typeof obj.then === 'function';
}

function isArray(arg) {
  return Object.prototype.toString.call(arg) === '[object Array]';
}

function isRegex(val) {
  if ((typeof val === 'undefined' ? 'undefined' : typeof val === 'undefined' ? 'undefined' : (0, _typeof3.default)(val)) === 'object' && (0, _getPrototypeOf2.default)(val).toString() === '/(?:)/') {
    return true;
  }

  return false;
}

function filterQuery() {
  var nodeList = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];
  var params = arguments[1];
  var negate = arguments.length <= 2 || arguments[2] === undefined ? false : arguments[2];

  if (typeof params === 'function') {
    return nodeList.filter(params);
  }

  var items = nodeList || [];

  return items.filter(function (node) {
    var matchesParam = (0, _keys2.default)(params).every(function (key) {
      var param = params[key];
      var value = (0, _lodash.result)(node, key);

      if (isRegex(param) && param.test(value)) {
        return true;
      }

      if (typeof param === 'string' && value === param) {
        return true;
      }

      if (typeof param === 'number' && value === param) {
        return true;
      }

      // treat normal arrays to search for some exact matches
      if (isArray(param) && ((0, _lodash.isString)(value) || (0, _lodash.isNumber)(value))) {
        return (0, _lodash.some)(param, function (val) {
          return val === value;
        });
      }

      if ((0, _lodash.isObject)(param) && (param.value || param.val) && (param.op || param.operator)) {
        return testWithOperator(param, value);
      }

      if ((0, _lodash.isObject)(param) && (0, _lodash.isObject)(value)) {
        return (0, _lodash.isMatch)(value, param);
      }
    });

    return negate ? !matchesParam : matchesParam;
  });
}

function abort(message) {
  console.log(message.red || message);
  process.exit(1);
}

function findPackageSync(packageName) {
  var root = arguments.length <= 1 || arguments[1] === undefined ? process.env.PWD : arguments[1];

  var findModules = require('find-node-modules');
  var path = require('path');

  var moduleDirectories = findModules(root, { relative: false });

  var directory = moduleDirectories.find(function (p) {
    var exists = pathExists((0, _path.join)(p, packageName));
    return exists;
  });

  if (!directory) {
    try {
      var resolvedPath = path.dirname(require.resolve(packageName));
      return resolvedPath;
    } catch (error) {
      //console.log('Error looking up package', packageName, error.message)
    }
  }

  return directory && path.resolve(path.join(directory, packageName));
}

function findPackage(packageName) {
  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  return new _promise2.default(function (resolve, reject) {
    var moduleDirectories = findModules(process.env.PWD, { relative: false });
    var directory = moduleDirectories.find(function (p) {
      var exists = pathExists((0, _path.join)(p, packageName));
      return exists;
    });

    if (!directory) {
      try {
        var resolvedPath = path.dirname(require.resolve(packageName));
        resolve(resolvedPath);
        return;
      } catch (error) {}
    }

    if (!directory) {
      reject(packageName);
    }

    var result = path.resolve(path.join(directory, packageName));

    if (result) {
      resolve(result);
    }
  });
}

function splitPath() {
  var p = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];

  return path.resolve(p).split(path.sep);
}

function skypagerBabel() {
  var presets = findPackageSync('babel-preset-skypager');

  try {
    presets ? require('babel-register')({ presets: presets }) : require('babel-register');
  } catch (error) {
    abort('Error loading the babel-register library. Do you have the babel-preset-skypager package?');
  }
}

function pathExists(fp) {
  var fs = require('fs');
  var fn = typeof fs.access === 'function' ? fs.accessSync : fs.statSync;

  try {
    fn(fp);
    return true;
  } catch (error) {
    return false;
  }
}

function findPackageManifest() {
  var cwd = arguments.length <= 0 || arguments[0] === undefined ? process.env.PWD : arguments[0];

  return require('findup-sync')('package.json', { cwd: cwd });
}

function loadManifestFromDirectory() {
  var cwd = arguments.length <= 0 || arguments[0] === undefined ? process.env.PWD : arguments[0];

  return require('findup-sync')('package.json', { cwd: cwd });
}

function testWithOperator(param, testValue) {
  var operator = param.operator || param.op;
  var checkValue = param.value || param.val;
  var testFn = operators[operator] || _lodash.isEqual;

  return testFn(testValue, checkValue);
}